*********************************************************************
*     PROGRAM dicom2phits                                           *
*                                                                   *
*         Created by T. Furuta on 2013/09/05                        *
*             Under collaboration between JAEA and RIKEN            *
*                                                                   *
*-------------------------------------------------------------------*
*  C functions "dicomlib.c" and                                     *
*  OS dependent C function "winlib.c" or "nonwinlib.c" are required *
*-------------------------------------------------------------------*
*                                                                   *
*     Purpose                                                       *
*         Convert dicom data to PHITS input                         *
*     Required files and a directry for run                         *
*         Conversion table file                                     *
*         DICOM file directory                                      *
*     Outputs                                                       *
*         Input file for PHITS (phits.inp)                          *
*         Include file containg material info (material.inp)        *
*         Include file containg universe info (universe.inp)        *
*         Include file containg voxel info (voxel.inp)              *
*         Include file containg library path info (libpath.inp)     *
*         [libpath.inp: only if it dose not exist]                  *
*                                                                   *
*         Last modified by T. Furuta on 2016/03/03                  *
*********************************************************************
      PROGRAM dicom2phits ! Main program
*********************************************************************
      use stdinout, only:OPENstdinout,CLOSEstdinout
      use readtable, only:READhvtable
      use readfile, only: READinputs,READheader,READfilenames,
     &     READdirectory,ALLOCATEfilenames,DEALLOCATEfilenames
      use createfile, only:CREATEfill,CREATEphitsinp
      implicit none
      integer :: nxnum,nynum,nznum
      integer :: nxc,nyc,nzc
      integer :: ixmin0,ixmax0,iymin0,iymax0,izmin0,izmax0
      integer :: ixnum,iynum,iznum,ixnum0,iynum0,iznum0
      integer :: izmin,izmax
      integer :: izstep,iorg,iparam
      integer :: nlist
      integer,allocatable :: mat(:,:,:),zlist(:)
      real(8) :: xdel,ydel,zdel,slope,hu0
      real(8) :: xorg,yorg,zorg
      real(8) :: xsh(3),xvec(3),yvec(3),zvec(3)
      real(8) :: xdel0,ydel0,zdel0
      character(LEN=200) :: Cconvtable,Cinput
      logical :: cgflag

      call OPENstdinout

      call READinputs(ixmin0,ixmax0,iymin0,iymax0,izmin0,izmax0,
     &     nxc,nyc,nzc,izstep,iorg,iparam,Cconvtable,Cinput)
      ixnum0=ixmax0-ixmin0+1
      iynum0=iymax0-iymin0+1
      iznum0=izmax0-izmin0+1

      call READhvtable(Cconvtable)

      call READdirectory(izmax0,nlist)
      nznum=nlist

      call ALLOCATEfilenames(nlist)

      call READfilenames(nlist)

      allocate(zlist(nznum))
      call SETzlist(izmin0,nznum,zlist)

      call READheader(izmin0,nxnum,nynum,xdel0,ydel0,zdel0,slope,hu0,
     &     xsh,xvec,yvec,zvec)

      call SETcoarse(nxc,nyc,nzc,
     &     ixnum0,iynum0,iznum0,ixnum,iynum,iznum,
     &     xdel0,ydel0,zdel0,xdel,ydel,zdel,cgflag)

      call SETzdirec(izstep,iorg,iznum0,izmin,izmax,zdel)

      allocate( mat(ixnum,iynum,iznum) )
      mat(:,:,:)=0
      call READCONV(ixmin0,iymin0,ixnum0,iynum0,izmin,izmax,
     &     ixnum,iynum,iznum,nxc,nyc,nzc,izstep,nxnum,nynum,
     &     nznum,zlist,mat)
      call CREATEfill(ixnum,iynum,iznum,mat,Cinput)
      deallocate( mat,zlist)

      call DEALLOCATEfilenames

      if(iorg.eq.0)then
        xorg=0.0d0
        yorg=0.0d0
        zorg=0.0d0
      else
        xorg=(ixmin0-1.5d0)*xdel0
        yorg=(iymin0-1.5d0)*ydel0
        zorg=-0.5d0*zdel0
      endif
      call CREATEphitsinp(iorg,iparam,ixnum,iynum,iznum,
     &     xdel,ydel,zdel,xorg,yorg,zorg,xsh,xvec,yvec,zvec,
     &     Cinput)

      call DSPparam(nxnum,nynum,nznum,xdel0,ydel0,zdel0,'ORIGINAL')

      call DSPparam(ixnum0,iynum0,iznum0,xdel0,ydel0,zdel0,'CLIPPED ')

      if(cgflag)then
         call DSPmessageCG
         call DSPparam(ixnum,iynum,iznum,xdel,ydel,zdel,'COASED  ')
      endif

      call CLOSEstdinout

      stop
      end

*********************************************************************
      subroutine SETzlist(izmin0,nznum,zlist)
*********************************************************************
      use readfile, only:Cfilenames,zloc
      implicit none
      integer,intent(in) :: izmin0,nznum
      integer,intent(out) :: zlist(nznum)
      integer :: iz,iz0,izmax0
      character(200) Ctmp
      real(8) ztmp

      do iz=1,nznum-1
        do iz0=iz,nznum
          if(zloc(iz0)<zloc(iz))then
            Ctmp=Cfilenames(iz)
            Cfilenames(iz)=Cfilenames(iz0)
            Cfilenames(iz0)=Ctmp
            ztmp=zloc(iz)
            zloc(iz)=zloc(iz0)
            zloc(iz0)=ztmp
          endif
        enddo
      enddo
      izmax0=izmin0+nznum-1
      iz=0
      do iz0=izmin0,izmax0
        iz=iz+1
        zlist(iz)=iz0
      enddo

      return
      end subroutine SETzlist

*********************************************************************
      subroutine SETcoarse(nxc,nyc,nzc,
     &     ixnum0,iynum0,iznum0,ixnum,iynum,iznum,
     &     xdel0,ydel0,zdel0,xdel,ydel,zdel,cgflag)
*********************************************************************
      use stdinout, only:stdout
      implicit none
      integer,intent(in) :: nxc,nyc,nzc
      integer,intent(inout) :: ixnum0,iynum0,iznum0
      integer,intent(out) :: ixnum,iynum,iznum
      real(8),intent(inout) :: xdel0,ydel0,zdel0
      real(8),intent(out) :: xdel,ydel,zdel
      logical,intent(out) :: cgflag
      if(nxc.gt.ixnum0)then
        write(stdout,'(a)')' ERROR: nxc is greter than PIXEL rows'
        write(stdout,'(a,i5,a,i5)')' nxc= ',nxc,'  nxnum=',ixnum0
        stop
      endif
      if(nyc.gt.iynum0)then
        write(stdout,'(a)')' ERROR: nyc is greter than PIXEL columns'
        write(stdout,'(a,i5,a,i5)')' nyc= ',nyc,'  nynum=',iynum0
        stop
      endif
      if(nzc.gt.iznum0)then
        write(stdout,'(a)')' ERROR: nzc is greter than # of DICOM files'
        write(stdout,'(a,i5,a,i5)')' nzc= ',nzc,'  nznum=',iznum0
        stop
      endif

      if(nxc.ne.1.or.nyc.ne.1.or.nzc.ne.1)then
         cgflag=.true.
      else
         cgflag=.false.
      endif
      ixnum=ixnum0/nxc
      ixnum0=ixnum*nxc
      xdel=xdel0*nxc
      iynum=iynum0/nyc
      iynum0=iynum*nxc
      ydel=ydel0*nyc
      iznum=iznum0/nzc
      iznum0=iznum*nzc
      zdel=zdel0*nzc
      return
      end subroutine SETcoarse
*********************************************************************
      subroutine SETzdirec(izstep,iorg,iznum,izmin,izmax,zdel)
*********************************************************************
      use stdinout, only:stdout
      implicit none
      integer,intent(inout) :: izstep
      integer,intent(in) :: iznum,iorg
      integer,intent(out) :: izmin,izmax
      real(8),intent(inout) :: zdel

      if(iorg.eq.0)then
        if(zdel.lt.0.0d0)then
         write(stdout,'(2a)')
     &        '**************************************************',
     &        '*******************'
          write(stdout,*)
     &        ' CAUTION: DICOM param SpacingBetweenSlices was set minus'
          write(stdout,*)
     &         ' CAUTION: Z direction is reversed'
          write(stdout,'(2a)')
     &         '**************************************************',
     &         '*******************'
          izstep=izstep*(-1)
          zdel=abs(zdel)
        endif
        if(izstep.eq.1)then
          izmin=1
          izmax=iznum
        elseif(izstep.eq.-1)then
         write(stdout,'(2a)')
     &        '**************************************************',
     &        '*******************'
          write(stdout,*)
     &        ' CAUTION: z direction set in descending order',
     &        ' of the slice number'
          write(stdout,'(2a)')
     &         '**************************************************',
     &         '*******************'
          izmin=iznum
          izmax=1
        else
          write(stdout,*)
     &         ' ERROR: Z direction should be +1 or -1 :: ',izstep
          stop
        endif
      else
        izstep=1
        izmin=1
        izmax=iznum
      endif
      return
      end subroutine SETzdirec
*********************************************************************
      subroutine READCONV(ixmin0,iymin0,ixnum0,iynum0,izmin,izmax,
     &     ixnum,iynum,iznum,nxc,nyc,nzc,izstep,nxnum,nynum,
     &     nznum,zlist,mat)
*********************************************************************
      use stdinout, only:stdout
      use readtable, only:CT2mat 
      use readfile, only:SETreadfile,Cpath,Cfilenames,Nfile
      implicit none
      integer,intent(in) :: ixmin0,iymin0
      integer,intent(in) :: ixnum0,iynum0
      integer,intent(in) :: izmin,izmax
      integer,intent(in) :: ixnum,iynum,iznum,izstep
      integer,intent(in) :: nxnum,nynum,nxc,nyc,nzc
      integer,intent(in) :: nznum,zlist(nznum)
      integer,intent(out) :: mat(ixnum,iynum,iznum)
      integer :: ixmax0,iymax0
      integer :: i,ix,iy,iz,iz0,matnum
      integer :: k,ntot
      logical :: Flow=.false.,Fup=.false.
      real(8) :: CTmin,CTmax
      real(8),allocatable:: hudata(:)
      real(8),allocatable:: hutmp(:,:)

      ixmax0=ixmin0+ixnum0-1
      iymax0=iymin0+iynum0-1

      if(ixmax0.gt.nxnum)then
        write(stdout,'(a,i4)')' ERROR: Nxmax should be <= ',nxnum
        stop
      elseif(iymax0.gt.nynum)then
        write(stdout,'(a,i4)')' ERROR: Nymax should be <= ',nynum
        stop
      endif

      ntot=nxnum*nynum
      allocate( hudata(nxnum*nynum),hutmp(ixnum,iynum) )
      hutmp(:,:)=0.0d0

      iz=0
      do i=izmin,izmax,izstep
        iz0=zlist(i)
        write(*,'(2a)')' Loading file = ',Cfilenames(iz0)(1:Nfile)
        call SETreadfile(iz0)
        call c_get_data(hudata,ntot,Cpath)

        do iy=1,nynum
          do ix=1,nxnum
            k=(iy-1)*nxnum+ix
            if(ix.ge.ixmin0.and.ix.le.ixmax0.and.
     &           iy.ge.iymin0.and.iy.le.iymax0)then
              hutmp((ix-ixmin0)/nxc+1,(iy-iymin0)/nyc+1)
     &             =hutmp((ix-ixmin0)/nxc+1,(iy-iymin0)/nyc+1)
     &             +hudata(k)/dble(nxc*nyc*nzc)
            endif
          enddo
        enddo
        if(mod(izstep*(i-izmin)+1,nzc).eq.0)then
          iz=iz+1
          do iy=1,iynum
            do ix=1,ixnum
              call CT2mat(hutmp(ix,iy),matnum,Flow,Fup,CTmin,CTmax)
              mat(ix,iy,iz) = matnum+5000
              hutmp(ix,iy)=0.0d0
            enddo
          enddo
        endif
      enddo

      deallocate( hudata,hutmp )

      if(Flow)then
        write(stdout,'(2a)')
     &       '**************************************************',
     &       '*******************'
        write(stdout,'(a)')
     &       ' WARNING: CT value lower than defined exists'
        write(stdout,'(a)')
     &       '     First universe is used for such a voxel'
        write(stdout,'(a,f10.2)')
     &       '           Lowest CT value =',CTmin
        write(stdout,'(2a)')
     &       '**************************************************',
     &       '*******************'
      endif
      if(Fup)then
        write(stdout,'(2a)')
     &       '**************************************************',
     &       '*******************'
        write(stdout,'(a)')
     &       ' WARNING: CT value higher than defined exists'
        write(stdout,'(a)')
     &       '     Last universe is used for such a voxel'
        write(stdout,'(a,f10.2)')
     &       '           Highest CT value =',CTmax
        write(stdout,'(2a)')
     &       '**************************************************',
     &       '*******************'
      endif
      return
      end subroutine READCONV

*********************************************************************
      subroutine DSPparam(nxnum,nynum,nznum,xdel,ydel,zdel,tag)
*********************************************************************
      use stdinout, only:stdout
      implicit none
      integer,intent(in) :: nxnum,nynum,nznum
      real(8),intent(in) :: xdel,ydel,zdel
      character(8),intent(in) :: tag

      write(stdout,'(3a)')'--------',tag,'--------'
      write(stdout,'(a,i5,a,i5,a,i5)')
     &     ' nx=',nxnum,' ny=',nynum,' nz=',nznum
      write(stdout,'(a,f12.5,a,f12.5,a,f12.5)')
     &     ' x=',xdel*nxnum,' y=',ydel*nynum,' z=',zdel*nznum
      return
      end subroutine DSPparam
*********************************************************************
      subroutine DSPmessageCG
*********************************************************************
      use stdinout, only:stdout
      implicit none
      write(stdout,'(2a)')'***************************',
     &     '******************************************'
      write(stdout,'(2a)')' CAUTION:',
     &     ' CLIPPED region may be modified by COASE GRANING'
      write(stdout,'(2a)')'***************************',
     &     '******************************************'
      return
      end subroutine DSPmessageCG
*********************************************************************
      subroutine DSPheader
*********************************************************************
      use stdinout, only:stdout,version
      implicit none

      write(stdout,'(2a)')'***************************',
     &     '******************************************'
      write(stdout,'(3a)')'*     ','PROGRAM dicom2phits',
     &     '                                           *'
      write(stdout,'(2a)')'*                          ',
     &     '                                         *'
      write(stdout,'(3a)')'*         ',
     &     'Created by T. Furuta on 2013/09/05',
     &     '                        *'
      write(stdout,'(3a)')'*             ',
     &     'Under collaboration between JAEA and RIKEN',
     &     '            *'
      write(stdout,'(2a)')'*                          ',
     &     '                                         *'
      write(stdout,'(2a)')'*--------------------------',
     &     '-----------------------------------------*'
      write(stdout,'(2a,f4.2,a)')'*                          '
     &     ,'Version =  ',version,'                          *'
      write(stdout,'(2a)')'*--------------------------',
     &     '-----------------------------------------*'
      write(stdout,'(2a)')'*                          ',
     &     '                                         *'
      write(stdout,'(3a)')'*     ','Purpose            ',
     &     '                                           *'
      write(stdout,'(3a)')'*         ',
     &     'Convert dicom data to PHITS input',
     &     '                         *'
      write(stdout,'(3a)')'*     ',
     &     'Required files and a directory for run',
     &     '                        *'
      write(stdout,'(3a)')'*         ',
     &     'Conversion table file',
     &     '                                     *'
      write(stdout,'(3a)')'*         ',
     &     'Dicom file directory',
     &     '                                      *'
      write(stdout,'(3a)')'*     ','Outputs',
     &     '                                                       *'
      write(stdout,'(3a)')'*         ',
     &     'Input file for PHITS (phits.inp)',
     &     '                          *'
      write(stdout,'(3a)')'*         ',
     &     'Include file containg material info (material.inp)',
     &     '        *'
      write(stdout,'(3a)')'*         ',
     &     'Include file containg universe info (universe.inp)',
     &     '        *'
      write(stdout,'(3a)')'*         ',
     &     'Include file containg voxel info (voxel.inp)',
     &     '              *'
      write(stdout,'(3a)')'*         ',
     &     'Include file containg library path info (libpath.inp)',
     &     '     *'
      write(stdout,'(3a)')'*         ',
     &     '[libpath.inp: only if it is necessary & dose not exist]',
     &     '   *'
      write(stdout,'(2a)')'*                          ',
     &     '                                         *'
      write(stdout,'(2a)')'*                          ',
     &     '                                         *'
      write(stdout,'(2a)')'***************************',
     &     '******************************************'
      return
      end subroutine DSPheader
*********************************************************************
