/***************************************************************************/
/*              C functions to deal with DICOM binary files                */
/*                                                                         */
/*                  Created by T. Furuta on 2013/10/10                     */
/*                     Under collaboration between JAEA and RIKEN          */
/*                                                                         */
/*                  Last modified by T. Furuta on 2014/05/09               */
/*-------------------------------------------------------------------------*/
/*   Main routine "dicom2phits.f" and                                      */
/*      OS dependent c function "winlib.c" or "nonwinlib.c" are required   */
/***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "dicomlib.h"
/********** Module variables ***********************************************/
int ifendian=0,imendian=0,imatch;
int nx,ny;
int nalloc,nstore,nhigh;
double zthick,xdel,ydel,zdel;
double hu0,slope;
double xsh[3],xvec[3],yvec[3];
char Fout[1024];
/********** Prototype decllaration *****************************************/
void c_get_header_(int *,int *,double *,double *,double *,double *,double *,
		   double *,double *,double *,char *,long);
void c_get_data_(double *,int *,char *, long);
void c_get_files_(int *, char *, long);

/********** Interface functions ********************************************/
void c_get_header_(int *nx0,int *ny0,double *xdel0,double *ydel0,double *zdel0,
		   double *hu00,double *slope0,
		   double *xsh0,double *xvec0,double *yvec0,
		   char *fname,long l)
{
  FILE *fp;
  if(open_file(&fp,fname,"rb")!=0)exit(-1);

  get_RTimageheader0(fp,&nx,&ny,&nalloc,
		     &xdel,&ydel,&zdel,
		     &hu0,&slope,
		     xsh,xvec,yvec);

  *nx0=nx;*ny0=ny;
  *xdel0=xdel;*ydel0=ydel;*zdel0=zdel;
  *hu00=hu0;*slope0=slope;
  xsh0[0]=xsh[0];xsh0[1]=xsh[1],xsh0[2]=xsh[2];
  xvec0[0]=xvec[0];xvec0[1]=xvec[1],xvec0[2]=xvec[2];
  yvec0[0]=yvec[0];yvec0[1]=yvec[1],yvec0[2]=yvec[2];
  
  if(close_file(fp)!=0)exit(-1);
  return;
}
/*-------------------------------------------------------------------------*/
void c_get_data_(double *data,int *ndata,char *fname,long l)
{
  FILE *fp;
  int i,ierr;
  float *data0=NULL;
  if(*ndata!=nx*ny){
    printf(" ERROR:: Contradicting number of data in DICOM: %d %d\n",
	   nx*ny,*ndata);
  }
  if(open_file(&fp,fname,"rb")!=0)exit(-1);

  data0=(float *)malloc(sizeof(float)*(*ndata));
  ierr=get_RTimagedata(fp,*ndata,nalloc,hu0,slope,data0);
  if(ierr!=0){
    printf(" ERROR:: DICOM file ended while scanning data\n");
    exit(-1);
  }
  for(i=0;i<*ndata;i++){
    data[i]=data0[i];
  }
  free(data0);
  if(close_file(fp)!=0)exit(-1);
  return;
}
/*-------------------------------------------------------------------------*/
/* void c_get_files_(int *nlist, char *path, long l){ */
/*   int i,j,ir; */
/*   FILE *fp2; */
/*   char buff[200]; */
/*   struct dirent **namelist; */

/*   ir=scandir(path, &namelist, NULL, alphasort); */
/*   if(ir == -1){ */
/*     printf(" ERROR:: Openning DICOM file directory: %s\n",path); */
/*     exit(-1); */
/*   } */
/*   if((fp2=fopen("tmp.txt","w"))==NULL){ */
/*     printf(" ERROR:: Openning temporary file: tmp.txt\n"); */
/*     exit(-1); */
/*   } */
/*   j=0; */
/*   for(i=0;i<ir;++i){ */
/*     if(strcmp(namelist[i]->d_name,".")&&strcmp(namelist[i]->d_name,"..")){ */
/*       strcpy(buff,path); */
/*       strcat(buff,namelist[i]->d_name); */
/*       if(check_file(buff)==0){ */
/* 	j++; */
/* 	fprintf(fp2,"%s\n",namelist[i]->d_name); */
/*       } */
/*     } */
/*     free(namelist[i]); */
/*   } */
/*   free(namelist); */
/*   *nlist=j; */
/*   fclose(fp2); */

/*   return; */
/* } */
/********** Internal functions **********/
